==> viper-0.15.tar.gz: viper-0.15/viperd.c <==
/*
** viperd.c
**
** The CMD Viper RAID monitoring daemon
**
** Copyright (c) 1998 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <math.h>
#include <signal.h>

#include <rts/rts.h>

#include "daemon.h"


char *event_eli_path = "/var/viper/viper.dat";
char *event_tmp_eli_path = "/var/viper/viper.tmp";
char *event_log_path = "/var/viper/viper.log";
char *viperd_pidfile = "/var/viper/viperd.pid";

int debug = 0;

int sleep_time = 600; /* Seconds between event log extraction */
char *send_mail_to = NULL;

FILE *event_log = NULL;

char **devtab = NULL;
int devidx = 0;
int devtabsize = 0;


time_t old_first_bt = 0;
int old_first = 0;
int old_last = 0;


int die = 0;

void
sigterm_handler()
{
    die = 1;
}


void
setup_event_log(RTS *rp)
{
    struct rts_event_log_information eli;
    struct rts_event_information ei;
    FILE *fp;
    int code;

    
    
    fp = fopen(event_eli_path, "r");
    if (fp)
    {
	fscanf(fp, "%d , %d , %u", &old_first, &old_last,
	       (unsigned int *) &old_first_bt);
	fclose(fp);
	return;
    }

    code = rts_read_event_log_information(rp, &eli);
    if (code)
    {
	rts_perror(stderr, "rts_read_event_log_information", rp);
	exit(1);
    }
    
    if (eli.status)
    {
	old_first = rts_w2i(eli.oldest_event_sequence_number);
	old_last = rts_w2i(eli.newest_event_sequence_number);
	    
	code = rts_read_event_information(rp, old_first, &ei);
	if (code)
	{
	    rts_perror(stderr, "rts_read_event_information", rp);
	    exit(1);
	}
	
	old_first_bt = rts_mktime(&ei.time_stamp);
    }
}
    

FILE *
open_sendmail(const char *subject)
{
    char buf[1024];
    FILE *fp;
	
    sprintf(buf, "/usr/lib/sendmail -t %s", send_mail_to);
    fp = popen(buf, "w");
    if (fp == NULL)
	return NULL;
    
    fprintf(fp, "To: %s\n", send_mail_to);
    fprintf(fp, "From: Viper Monitoring Daemon <root>\n");
    fprintf(fp, "Subject: %s\n\n", subject);

    return fp;
}


FILE *
open_sendmail_event(const char *event)
{
    char buf[256];

    strcpy(buf, event);
    strcat(buf, " Event");
    return open_sendmail(buf);
}


void
do_event_log(RTS *rp)
{
    FILE *mail_fp = NULL;
    FILE *fp;
    struct rts_event_log_information eli;
    struct rts_event_information first_ei, ei;
    time_t first_bt;
    int i, code, first, last;

    
    code = rts_read_event_log_information(rp, &eli);
    if (code)
    {
	rts_perror(stderr, "rts_read_event_log_information", rp);
	exit(1);
    }

    if (eli.status)
    {
	first = rts_w2i(eli.oldest_event_sequence_number);
	last = rts_w2i(eli.newest_event_sequence_number);
	
	code = rts_read_event_information(rp, first, &first_ei);
	if (code)
	{
	    rts_perror(stderr, "rts_read_event_information", rp);
	    exit(1);
	}

	first_bt = rts_mktime(&first_ei.time_stamp);


	if (old_first == first &&
	    difftime(old_first_bt, first_bt) == 0)
	{
	    /* Same start of the event log - no Flush */
	    i = old_last+1;
	}
	else
	{
	    /* Event log was flushed - start from the beginning */
	    i = first;
	    
	    if (debug)
		fprintf(stderr, "Event log flushed\n");
		       
	    fp = open_sendmail_event("Pseudo");
	    fprintf(fp, "Event Type:\tEvent log flushed.\n");
	    pclose(fp);
	}

	if (send_mail_to && (last - i) > 0)
	{
	    if (debug)
		fprintf(stderr, "Multiple new events: %d\n", (last-i)+1);
	    
	    mail_fp = open_sendmail("Multiple Events");
	}
	
	while (i <= last)
	{
	    code = rts_read_event_information(rp, i, &ei);
	    if (code)
	    {
		rts_perror(stderr, "rts_read_event_information", rp);
		exit(1);
	    }
	
	    fputs("------------------------------------------------------------------------------\n", event_log);
	    rts_print_event(event_log, &ei);
	    fflush(event_log);

	    syslog(LOG_NOTICE, "%s Event", 
			rts_strevtype(ei.event_type));
	    
	    if (send_mail_to)
	    {
		if (mail_fp == NULL)
		{
		    mail_fp = open_sendmail_event(rts_strevtype(ei.event_type));
		    rts_print_event(mail_fp, &ei);
		    pclose(mail_fp);
		    mail_fp = NULL;
		}
		else
		{
		    rts_print_event(mail_fp, &ei);
		    if (i < last)
			fputs("------------------------------------------------------------------------------\n", mail_fp);
		}
	    }
	    
	    ++i;
	}

	if (mail_fp != NULL)
	{
	    pclose(mail_fp);
	    mail_fp = NULL;
	}
	
	old_first = first;
	old_first_bt = first_bt;
	old_last = last;

	fp = fopen(event_tmp_eli_path, "w");
	if (fp == NULL)
	{
	    perror("fopen()");
	    exit(1);
	}
	
	if (fprintf(fp, "%d, %d, %u\n",
		    old_first,
		    old_last,
		    (unsigned int) old_first_bt) < 0)
	{
	    syslog(LOG_ERR, "fprintf: %m");
	    fclose(fp);
	    return;
	}

	rename(event_tmp_eli_path, event_eli_path);
	fclose(fp);
    }
}


void
do_time(RTS *rp)
{
    int code;

    time_t c_t, o_t;

    code = rts_read_date_and_time(rp, &c_t);
    if (code)
    {
	rts_perror(stderr, "rts_read_date_and_time", rp);
	exit(1);
    }
    
    time(&o_t);
    
    if (debug)
    {
	fprintf(stderr, "Our time:   %s\n", ctime(&o_t));
	fprintf(stderr, "Viper time: %s\n", ctime(&c_t));
    }
    
    if (fabs(difftime(o_t, c_t)) > 5)
    {
	
	code = rts_begin_session(rp, 0);
	if (code)
	{
	    rts_perror(stderr, "rts_begin_session", rp);
	    exit(1);
	}
	
	code = rts_write_date_and_time(rp, o_t);
	if (code)
	{
	    rts_perror(stderr, "rts_write_date_and_time", rp);
	    rts_end_session(rp);
	    exit(1);
	}

	
	code = rts_end_session(rp);
	if (code)
	{
	    rts_perror(stderr, "rts_end_session", rp);
	    exit(1);
	}
	
	syslog(LOG_NOTICE, "Time set on Viper");
    }
}


void
do_stats(RTS *rp)
{
    /* Later */
}


void
usage(FILE *fp)
{
    fprintf(fp, "Usage: viperd [options] <device>\n");
    fprintf(fp, "Options:\n");
    fprintf(fp, "\t-d          Debug mode\n");
    fprintf(fp, "\t-m<addr>    Mail address\n");
    fprintf(fp, "\t-t<time>    Poll time (in seconds)\n");
    fprintf(fp, "\t-h          Display this information\n");
}


int
main(int argc, char *argv[])
{
    RTS *rp;
    int c;
    struct sigaction sa;

    sigaction(SIGTERM, NULL, &sa);
    sa.sa_handler = sigterm_handler;
    sa.sa_flags = 0;
    sigaction(SIGTERM, &sa, NULL);

    while ((c = getopt(argc, argv, "dhm:t:")) != -1)
	switch (c)
	{
	  case 'd':
	    debug = 1;
	    break;
	    
	  case 'm':
	    send_mail_to = strdup(optarg);
	    break;

	  case 't':
	    sleep_time = atoi(optarg);
	    break;
	    
	  case 'h':
	    usage(stdout);
	    exit(0);
	    
	  default:
	    usage(stderr);
	    exit(1);
	}
    
    if (optind >= argc)
    {
	usage(stderr);
	exit(1);
    }

    if (!debug)
	become_daemon();
    
    openlog("viperd", LOG_PID|LOG_CONS, LOG_LOCAL4);
    syslog(LOG_INFO, "Started");

    pidfile_create(viperd_pidfile);
    
    event_log = fopen(event_log_path, "a");
    if (event_log == NULL)
    {
	if (debug)
	{
	    fprintf(stderr, "%s: fopen: ", argv[0]);
	    perror(event_log_path);
	}
	else
	    syslog(LOG_ERR, "fopen: %s: %m", event_log_path);
	exit(1);
    }

    fputs("------------------------------------------------------------------------------\n", event_log);
    fprintf(event_log, "Viperd Started\n\n\n");
    fflush(event_log);
    
    rp = rts_open(argv[optind]);
    if (rp == NULL)
    {
	if (debug)
	{
	    fprintf(stderr, "%s: rts_open: ", argv[0]);
	    perror(argv[optind]);
	}
	else
	    syslog(LOG_ERR, "rts_open: %s: %m", argv[optind]);
	exit(1);
    }

    setup_event_log(rp);
    
    while (!die)
    {
	do_time(rp);
	do_event_log(rp);
	do_stats(rp);
	
	sleep(sleep_time);
    }

    syslog(LOG_INFO, "Terminating");
    unlink(viperd_pidfile);
    return 1;
}

==> viper-0.17.tar.gz: viper-0.17/viperd.c <==
/*
** viperd.c
**
** The CMD Viper RAID monitoring daemon
**
** Copyright (c) 1998-1999 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <math.h>
#include <signal.h>

#include <rts/rts.h>

#include "daemon.h"

extern char version[];

char *name = NULL;

char *var_path = "/var/viper";

char *event_eli_file = "viper.dat";
char *event_tmp_eli_file = "viper.tmp";

char *event_log_file = "viper.log";
char *daemon_pid_file = "viperd.pid";

char event_eli_path[1024];
char event_tmp_eli_path[1024];
char event_log_path[1024];
char daemon_pid_path[1024];

int debug = 0;

int sleep_time = 600; /* Seconds between event log extraction */
char *send_mail_to = NULL;

FILE *event_log = NULL;

char **devtab = NULL;
int devidx = 0;
int devtabsize = 0;


time_t old_first_bt = 0;
int old_first = 0;
int old_last = 0;


int die = 0;

void
sigterm_handler()
{
    die = 1;
}


void
setup_event_log(RTS *rp)
{
    struct rts_event_log_information eli;
    struct rts_event_information ei;
    FILE *fp;
    int code;

    
    
    fp = fopen(event_eli_path, "r");
    if (fp)
    {
	fscanf(fp, "%d , %d , %u", &old_first, &old_last,
	       (unsigned int *) &old_first_bt);
	fclose(fp);
	return;
    }

    code = rts_read_event_log_information(rp, &eli);
    if (code)
    {
	rts_perror(stderr, "rts_read_event_log_information", rp);
	exit(1);
    }
    
    if (eli.status)
    {
	old_first = rts_w2i(eli.oldest_event_sequence_number);
	old_last = rts_w2i(eli.newest_event_sequence_number);
	    
	code = rts_read_event_information(rp, old_first, &ei);
	if (code)
	{
	    rts_perror(stderr, "rts_read_event_information", rp);
	    exit(1);
	}
	
	old_first_bt = rts_mktime(&ei.time_stamp);
    }
}
    

FILE *
open_sendmail(const char *subject)
{
    char buf[1024];
    FILE *fp;

    if (send_mail_to == NULL)
	return NULL;
    
    sprintf(buf, "/usr/lib/sendmail -t %s", send_mail_to);
    fp = popen(buf, "w");
    if (fp == NULL)
	return NULL;
    
    fprintf(fp, "To: %s\n", send_mail_to);
    if (name)
	fprintf(fp, "From: Viper Monitoring Daemon (%s) <root>\n", name);
    else
	fprintf(fp, "From: Viper Monitoring Daemon <root>\n");
    
    fprintf(fp, "Subject: %s\n\n", subject);

    return fp;
}


FILE *
open_sendmail_event(const char *event)
{
    char buf[256];

    strcpy(buf, event);
    strcat(buf, " Event");
    return open_sendmail(buf);
}


void
do_event_log(RTS *rp)
{
    FILE *mail_fp = NULL;
    FILE *fp;
    struct rts_event_log_information eli;
    struct rts_event_information first_ei, ei;
    time_t first_bt;
    int i, code, first, last;

    
    code = rts_read_event_log_information(rp, &eli);
    if (code)
    {
	rts_perror(stderr, "rts_read_event_log_information", rp);
	exit(1);
    }

    if (eli.status)
    {
	first = rts_w2i(eli.oldest_event_sequence_number);
	last = rts_w2i(eli.newest_event_sequence_number);
	
	code = rts_read_event_information(rp, first, &first_ei);
	if (code)
	{
	    rts_perror(stderr, "rts_read_event_information", rp);
	    exit(1);
	}

	first_bt = rts_mktime(&first_ei.time_stamp);


	if (old_first == first &&
	    difftime(old_first_bt, first_bt) == 0)
	{
	    /* Same start of the event log - no Flush */
	    i = old_last+1;
	}
	else
	{
	    /* Event log was flushed - start from the beginning */
	    i = first;
	    
	    if (debug)
		fprintf(stderr, "Event log flushed\n");
		       
	    fp = open_sendmail_event("Pseudo");
	    if (fp != NULL)
	    {
		fprintf(fp, "Event Type:\tEvent log flushed.\n");
		pclose(fp);
	    }
	}

	if (send_mail_to && (last - i) > 0)
	{
	    if (debug)
		fprintf(stderr, "Multiple new events: %d\n", (last-i)+1);
	    
	    mail_fp = open_sendmail("Multiple Events");
	}
	
	while (i <= last)
	{
	    code = rts_read_event_information(rp, i, &ei);
	    if (code)
	    {
		rts_perror(stderr, "rts_read_event_information", rp);
		exit(1);
	    }

	    if (name)
		syslog(rts_event2severity(&ei), "%s: %s",
		       name, rts_str_event(&ei));
	    else
		syslog(rts_event2severity(&ei), "%s",
		       rts_str_event(&ei));
	    
	    fputs("------------------------------------------------------------------------------\n", event_log);
	    rts_print_event(event_log, &ei);
	    fflush(event_log);

	    if (send_mail_to)
	    {
		if (mail_fp == NULL)
		{
		    mail_fp = open_sendmail_event(rts_strevtype(ei.event_type));
		    if (mail_fp)
		    {
			rts_print_event(mail_fp, &ei);
			pclose(mail_fp);
		    }
		    mail_fp = NULL;
		}
		else
		{
		    rts_print_event(mail_fp, &ei);
		    if (i < last)
			fputs("------------------------------------------------------------------------------\n", mail_fp);
		}
	    }
	    
	    ++i;
	}

	if (mail_fp != NULL)
	{
	    pclose(mail_fp);
	    mail_fp = NULL;
	}
	
	old_first = first;
	old_first_bt = first_bt;
	old_last = last;

	fp = fopen(event_tmp_eli_path, "w");
	if (fp == NULL)
	{
	    perror("fopen()");
	    exit(1);
	}
	
	if (fprintf(fp, "%d, %d, %u\n",
		    old_first,
		    old_last,
		    (unsigned int) old_first_bt) < 0)
	{
	    syslog(LOG_ERR, "fprintf: %m");
	    fclose(fp);
	    return;
	}

	rename(event_tmp_eli_path, event_eli_path);
	fclose(fp);
    }
}


void
do_time(RTS *rp)
{
    int code;

    time_t c_t, o_t;

    code = rts_read_date_and_time(rp, &c_t);
    if (code)
    {
	perror("mktime");
	rts_perror(stderr, "rts_read_date_and_time", rp);
	exit(1);
    }
    
    time(&o_t);
    
    if (debug)
    {
	fprintf(stderr, "Our time:   %s\n", ctime(&o_t));
	fprintf(stderr, "Viper time: %s\n", ctime(&c_t));
    }
    
    if (fabs(difftime(o_t, c_t)) > 5)
    {
	
	code = rts_begin_session(rp, 0);
	if (code)
	{
	    rts_perror(stderr, "rts_begin_session", rp);
	    exit(1);
	}
	
	code = rts_write_date_and_time(rp, o_t);
	if (code)
	{
	    rts_perror(stderr, "rts_write_date_and_time", rp);
	    rts_end_session(rp);
	    exit(1);
	}

	
	code = rts_end_session(rp);
	if (code)
	{
	    rts_perror(stderr, "rts_end_session", rp);
	    exit(1);
	}
	
	syslog(LOG_NOTICE, "Time set on Viper");
    }
}


void
do_stats(RTS *rp)
{
    /* Later */
}


void
usage(FILE *fp)
{
    fprintf(fp, "Usage: viperd [options] <device>\n");
    fprintf(fp, "Options:\n");
    fprintf(fp, "\t-V          Print server version\n");
    fprintf(fp, "\t-d          Debug mode\n");
    fprintf(fp, "\t-m<addr>    Mail address\n");
    fprintf(fp, "\t-t<time>    Poll time (in seconds)\n");
    fprintf(fp, "\t-h          Display this information\n");
}


int
main(int argc, char *argv[])
{
    RTS *rp;
    int c, code;
    struct sigaction sa;
    struct rts_vendor_parameters vpb;
    

    sigaction(SIGTERM, NULL, &sa);
    sa.sa_handler = sigterm_handler;
    sa.sa_flags = 0;
    sigaction(SIGTERM, &sa, NULL);

    while ((c = getopt(argc, argv, "Vdhm:t:n:")) != -1)
	switch (c)
	{
	  case 'V':
	    printf("Viperd %s, built on %s %s\n",
		   version, __DATE__, __TIME__);
	    printf("RTS Version: %d\n", RTS_VERSION);
	    exit(0);

	  case 'n':
	    name = strdup(optarg);
	    break;
	    
	  case 'd':
	    debug = 1;
	    break;
	    
	  case 'm':
	    send_mail_to = strdup(optarg);
	    break;

	  case 't':
	    sleep_time = atoi(optarg);
	    break;
	    
	  case 'h':
	    usage(stdout);
	    exit(0);
	    
	  default:
	    usage(stderr);
	    exit(1);
	}
    
    if (optind >= argc)
    {
	usage(stderr);
	exit(1);
    }

    if (!debug)
	become_daemon();

    sprintf(event_eli_path, "%s/%s/%s",
	    var_path, name ? name : "",
	    event_eli_file);
    
    sprintf(event_tmp_eli_path, "%s/%s/%s",
	    var_path, name ? name : "",
	    event_tmp_eli_file);
    
    sprintf(event_log_path, "%s/%s/%s",
	    var_path, name ? name : "",
	    event_log_file);
    
    sprintf(daemon_pid_path, "%s/%s/%s",
	    var_path, name ? name : "",
	    daemon_pid_file);
    
    openlog("viperd", LOG_PID|LOG_CONS, LOG_LOCAL4);
    syslog(LOG_INFO, "Started for viper: %s", name);

    pidfile_create(daemon_pid_path);
    
    event_log = fopen(event_log_path, "a");
    if (event_log == NULL)
    {
	if (debug)
	{
	    fprintf(stderr, "%s: fopen: ", argv[0]);
	    perror(event_log_path);
	}
	else
	    syslog(LOG_ERR, "fopen: %s: %m", event_log_path);
	exit(1);
    }

    fputs("------------------------------------------------------------------------------\n", event_log);
    fprintf(event_log, "Viperd version %s started\n\n\n", version);
    fflush(event_log);
    
    rp = rts_open(argv[optind]);
    if (rp == NULL)
    {
	if (debug)
	{
	    fprintf(stderr, "%s: rts_open: ", argv[0]);
	    perror(argv[optind]);
	}
	else
	    syslog(LOG_ERR, "rts_open: %s: %m", argv[optind]);
	exit(1);
    }

    code = rts_read_vendor_parameters(rp, &vpb);
    if (code != 0)
    {
	syslog(LOG_ERR, "rts_read_vendor_parameters: %m");
	exit(1);
    }

    printf("Display Vendor: %.16s\n", vpb.system_display_vendor);
    printf("Display Model:  %.16s\n", vpb.system_display_model);
    printf("Inquiry Vendor: %.8s\n", vpb.inquiry_response_vendor);
    printf("Inquiry Model:  %.16s\n", vpb.inquiry_response_model);
    
#if RTS_VERSION >= 2
    printf("%% Write back cache: %d\n", vpb.percent_write_back_cache);
#endif

    setup_event_log(rp);
    
    while (!die)
    {
	do_time(rp);
	do_event_log(rp);
	do_stats(rp);
	
	sleep(sleep_time);
    }

    syslog(LOG_INFO, "Terminating");
    unlink(daemon_pid_path);
    return 1;
}
